'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _hook = require('./hook');

var _hook2 = _interopRequireDefault(_hook);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A hook manager.
 */
class HookManager extends Map {
  /**
   * Add an handler to the given hook.
   */
  use(name, handler) {
    const hook = this.get(name) || new _hook2.default();

    hook.add(handler);
    this.set(name, hook);
  }

  /**
   * Run the given hook.
   */
  async run(context, name, ...args) {
    const hook = this.get(name);

    if (hook) {
      return hook.run(context, ...args);
    }

    return true;
  }
}
exports.default = HookManager;