'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _eventemitter = require('eventemitter2');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _orchestrator = require('orchestrator');

var _orchestrator2 = _interopRequireDefault(_orchestrator);

var _plugin = require('./plugin');

var _plugin2 = _interopRequireDefault(_plugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A plugin manager.
 */
class PluginManager extends _eventemitter2.default {

  /**
   * Construct a new plugin manager.
   */


  /**
   * Plugin registrations.
   *
   * @private
   */

  /**
   * The main application instance.
   */
  constructor(app) {
    super();

    this.registrations = new Set();
    this.plugins = new Map();
    this.app = app;
    this.logger = this.app.logger.child({ module: 'plugin-manager' });

    // Dispatch all events to plugins.
    this.onAny((event, ...args) => {
      this.plugins.forEach(plugin => {
        plugin.emit(event, ...args);
      });
    });
  }

  /**
   * Register a plugin.
   */


  /**
   * The loaded plugins.
   *
   * @private
   */


  /**
   * The logger instance.
   *
   * @private
   */
  registerPlugin(plugin, options = {}) {
    const { attributes } = plugin.register;
    const registration = {
      options,
      register: plugin.register,
      name: attributes.name || attributes.pkg.name,
      enforce: attributes.enforce,
      dependencies: attributes.dependencies || []
    };

    this.registrations.add(registration);
    this.logger.debug(`Plugin "${registration.name}" registered`);
  }

  /**
   * Load a plugin.
   */
  async loadPlugin(registration) {
    const {
      register, name, dependencies, options
    } = registration;
    const plugin = new _plugin2.default(this.app, { name, dependencies });

    await register(plugin, options);

    this.plugins.set(plugin.name, plugin);
    this.logger.debug(`Plugin "${plugin.name}" loaded`);

    return plugin;
  }

  /**
   * Load all plugins.
   */
  loadPlugins() {
    const orchestrator = new _orchestrator2.default();
    const preLoaders = [];
    const loaders = [];
    const postLoaders = [];

    this.registrations.forEach(registration => {
      const { name, dependencies, enforce } = registration;

      if (enforce === 'pre') {
        preLoaders.push(name);
      } else if (enforce === 'post') {
        postLoaders.push(name);
      } else {
        loaders.push(name);
      }

      orchestrator.add(name, dependencies, () => this.loadPlugin(registration));
    });

    return _bluebird2.default.fromCallback(callback => {
      orchestrator.start(preLoaders, loaders, postLoaders, callback);
    });
  }

  /**
   * Get all loaded plugins.
   */
  getPlugins() {
    return Array.from(this.plugins.values());
  }

  /**
   * Get a loaded plugin.
   */
  getPlugin(name) {
    return this.plugins.get(name);
  }

  /**
   * Clear all loaded plugins.
   */
  async clearPlugins() {
    await _bluebird2.default.map(this.getPlugins(), plugin => plugin.destroy());
    this.plugins.clear();
  }
}
exports.default = PluginManager;