'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _eventemitter = require('eventemitter2');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _application = require('./application');

var _application2 = _interopRequireDefault(_application);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A plugin.
 */
class Plugin extends _eventemitter2.default {

  /**
   * The Discord client.
   */


  /**
   * Plugin name.
   */

  /**
   * Application instance.
   */
  get client() {
    return this.app.client;
  }

  /**
   * The hook manager.
   */


  /**
   * Plugin dependencies.
   */


  /**
   * The logger instance.
   *
   * @private
   */
  get hookManager() {
    return this.app.hookManager;
  }

  /**
   * Construct a new plugin.
   */
  constructor(app, params) {
    super();

    this.app = app;
    this.logger = this.app.logger.child({ module: 'plugin' });
    this.name = params.name;
    this.dependencies = params.dependencies;
  }

  /**
   * Destroy the plugin.
   */
  async destroy() {
    this.logger.debug('Destroying plugin...');
    await this.emitAsync('destroy');
  }

  /**
   * Get an existing plugin.
   */
  getPlugin(name) {
    return this.app.pluginManager.getPlugin(name);
  }

  /**
   * Extend with a new property.
   */
  decorate(type, name, value) {
    switch (type) {
      case 'app':
        this.app.decorate(name, value);
        break;

      default:
        Object.defineProperty(Plugin.prototype, name, { value });
    }
  }

  /**
   * Expose a property.
   */
  expose(name, value) {
    Object.defineProperty(this, name, { value });
  }

  /**
   * Expose a method.
   */
  method(name, handler) {
    Object.defineProperty(this, name, { value: handler.bind(this) });
  }

  /**
   * Register a new hook handler.
   */
  hook(name, handler) {
    this.hookManager.use(name, handler.bind(this));
  }

  /**
   * Register a list of methods.
   */
  methods(methods) {
    (0, _lodash.forEach)(methods.default || methods, (handler, name) => {
      this.method(name, handler);
    });
  }

  /**
   * Schedule a repeated execution.
   */
  setInterval(callback, delay, ...args) {
    return this.app.setInterval(callback, delay, ...args);
  }

  /**
   * Schedule a one-time execution.
   */
  setTimeout(callback, delay, ...args) {
    return this.app.setTimeout(callback, delay, ...args);
  }

  /**
   * Clear an interval.
   */
  clearInterval(interval) {
    this.app.clearInterval(interval);
  }

  /**
   * Clear a timeout.
   */
  clearTimeout(timeout) {
    this.app.clearTimeout(timeout);
  }
}
exports.default = Plugin;